#ifndef _LINUX_CAN_H
#define _LINUX_CAN_H

#include "can-fifo.h"

#define CAN_MAX_ENDPOINTS	CONFIG_CAN_PHYS_ENDPOINT_NUM

#define CAN_SETBAUDRATE				0x01
#define CAN_IOCHIPRESET				0x02
#define CAN_MODE						0x03

struct can_endpoint_t {
	int minor;	/* Minor number of can device */
	atomic_t send_ready_flag;	
	char name[8];	/* Device name */
	void *chip;	/* Point to specific chip  */
	can_fifo_t rxfifo;	/* Rx FIFO */
	can_fifo_t txfifo;	/* Tx FIFO */
	wait_queue_head_t read_queue;
	wait_queue_head_t write_queue;
	void (*set_bitrate)(struct can_endpoint_t *endpoint, int bitrate);
	void (*chip_reset)(struct can_endpoint_t *endpoint);
	void (*set_mode)(struct can_endpoint_t *endpoint,int mode);		
	int (*initiate2write)(struct can_endpoint_t *endpoint);
};


struct can_dev_client {
	int uid;
	u32 magic_num;
	int reserved;
	struct can_endpoint_t * endpoint;
};

extern struct class *can_dev_class;
extern can_frame_t * can_fifo_get(can_fifo_t *fifo, int *retval);
can_frame_t * can_endpoint_getframe(struct can_endpoint_t *endpoint, int * retval);
extern int can_endpoint_adduser(struct can_endpoint_t *endpoint);
extern int can_endpoint_deluser(struct can_endpoint_t *endpoint, int uid);
extern void can_endpoint_writeframe(struct can_endpoint_t *endpoint) ;
extern can_frame_t * can_endpoint_readframe(struct can_endpoint_t *endpoint, int uid, int  *retval);
extern struct can_endpoint_t * can_endpoint_init(int factorTx2Rx);
extern struct can_endpoint_t *can_endpoint_get_by_minor(unsigned index);
extern void can_endpoint_free(struct can_endpoint_t *endpoint);
extern int can_endpoint_register(struct can_endpoint_t *endpoint) ;
extern void can_endpoint_unregister(struct can_endpoint_t *endpoint) ;
#endif /* _LINUX_CAN_H */
