#ifndef _LINUX_CANFIFO_H
#define _LINUX_CANFIFO_H
#include <linux/spinlock.h>

/* Maximum number of connected clients */
#define CAN_MAX_USER		8
#define CAN_MAX_USER_BIT	((1<<CAN_MAX_USER) - 1)

typedef struct can_frame {
	u32 id;    	 /* CAN ID */
	u8 efi;		/* Extended format identifier  bit */
	u8 data_len;    	
	u8 rtr;     	/* Remote transmission request bit */
	u8 data[8]; 	/* Message */
	struct timeval tv; /* Timestamp for frame package */
}can_frame_t;

/* CAN FIFO structure */
typedef struct can_fifo {
	spinlock_t lock;	
	unsigned long flags;		/* Backup for interrupt status */
	/*read overrun status bit, so CAN_MAX_ENDPOINTS should be less than 32-bit*/		
	int rd_overrun_bit; 
	u32 user_counter;	/*can endpoint user counter */
	int size;	
	int start;					/* Start offset */	
	int end[CAN_MAX_USER];	/* End offset */
	can_frame_t *data;	/* Buffer loop-pool */	
}can_fifo_t;

static void inline can_fifo_lock(can_fifo_t *fifo)
{
	spin_lock_irqsave(&fifo->lock, fifo->flags);
}

static void inline can_fifo_unlock(can_fifo_t *fifo)
{
	spin_unlock_irqrestore(&fifo->lock, fifo->flags);
}

extern int can_fifo_init(can_fifo_t *fifo, int size);
extern int can_fifo_adduser(can_fifo_t *fifo);
extern int can_fifo_deluser(can_fifo_t *fifo, int id);
extern int can_fifo_push(can_fifo_t *fifo,can_frame_t *frame);
extern can_frame_t * can_fifo_pop(can_fifo_t *fifo, int id, int  *retval);
#endif /* _LINUX_CANFIFO_H*/
