/*
 * Can of_devices probe and chip function
 *
 * Re-written by Zumeng.Chen <zumeng.chen@windriver.com>
 * Based on peak-linux-driver.3.17.tar.gz.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/device.h>
#include <asm/io.h>
#include <asm/prom.h>
#include <asm/of_device.h>
#include <asm/of_platform.h>
#include <linux/interrupt.h>
#include <linux/delay.h>

#include "can-base.h"
#include "can-mpc52xx-of.h" 



/* -------------------chip function  start--------------------------------------*/
/* 
 * Register a mpc52xx chip with the chip layer.
 * And there may be ugly deadloop here, it should be fixed at somepoint
 */
static int mscan_init(struct can_mpc52xx_chip_t  * chip)
{
	struct mpc52xx_mscan * regs = chip->regs;

#ifdef CONFIG_CAN_DEBUG
	printk(KERN_INFO "MSCAN: Initializing CAN chip bitrate=0x%x:\n",
		chip->wBTR0BTR1);
#endif

	/* Enable MSCAN module. */
	regs->canctl1 |= MPC52xx_MSCAN_CANE;
	udelay(100);

	/* Switch to sleep mode */
	regs->canctl0 |= MPC52xx_MSCAN_SLPRQ;	
	while (!(regs->canctl1 & MPC52xx_MSCAN_SLPAK));
		
	/* Switch to initialization mode. */
	regs->canctl0 |= MPC52xx_MSCAN_INITRQ;
	while (!(regs->canctl1 & MPC52xx_MSCAN_INITAK));
	
	/* Set listen-only mode if needed. */
	if (chip->ucListenOnly)
		regs->canctl1 |= MPC52xx_MSCAN_LISTEN;
	else
		regs->canctl1 &= ~MPC52xx_MSCAN_LISTEN;

	/* Set endpoint timings.	 */
	regs->canbtr0 = (chip->wBTR0BTR1 >> 8);
	regs->canbtr1 = (u8)chip->wBTR0BTR1;

	/*Set internal timer */
	regs->canctl0 |= MPC52xx_MSCAN_TIME;

	/*IP endpoint as clock source. It looks this bit is not used in lite5200b*/
	regs->canctl1 &= (~MPC52xx_MSCAN_CLKSRC);

	if(chip->loopbackmode)
		regs->canctl1 |= MPC52xx_MSCAN_LOOPB;
	else 
		regs->canctl1 &= ~MPC52xx_MSCAN_LOOPB;
	
	/* Configure MSCAN to accept all incoming messages. */
	regs->canidar0 = regs->canidar1 = 0x00;
	regs->canidar2 = regs->canidar3 = 0x00;
	regs->canidmr0 = regs->canidmr1 = 0xFF;
	regs->canidmr2 = regs->canidmr3 = 0xFF;
	regs->canidar4 = regs->canidar5 = 0x00;
	regs->canidar6 = regs->canidar7 = 0x00;
	regs->canidmr4 = regs->canidmr5 = 0xFF;
	regs->canidmr6 = regs->canidmr7 = 0xFF;
	regs->canidac &= ~(MPC52xx_MSCAN_IDAM0 | MPC52xx_MSCAN_IDAM1);

	/* Switch back to normal mode.	 */
	regs->canctl0 &= ~MPC52xx_MSCAN_INITRQ;
	regs->canctl0 &= ~MPC52xx_MSCAN_SLPRQ;
	while((regs->canctl1 & MPC52xx_MSCAN_INITAK) ||
	      (regs->canctl1 & MPC52xx_MSCAN_SLPAK));
	
	/* Enable receive interrupts.	 */
	regs->canrier |= MPC52xx_MSCAN_OVRIE | MPC52xx_MSCAN_RXFIE;	
	
#ifdef CONFIG_CAN_DEBUG
	printk(KERN_INFO "MSCAN: bitrate= 0x%x %x ctl1=0x%x\n",
		regs->canbtr0,regs->canbtr1,regs->canctl1);
#endif
	return 0;
}



/* Read CAN data from the controller. Called from the interrupt handler on
  * receive interrupt. */
static int mscan_read(struct can_mpc52xx_chip_t *chip, can_frame_t *frame)
{
	struct mpc52xx_mscan * regs = chip->regs;
	volatile ID        local;
	u8                 ucLen;
	int                i;
	int                result  = 0;

#ifdef CONFIG_CAN_DEBUG
	printk("%s: mscan_read() \n", DEVICE_NAME);
#endif 
	frame->rtr = 0;
	
	/* Filter out extended messages in non extended mode. */
	if (!chip->bExtended && (regs->canrxfg.idr[1] & BUFFER_EXTENDED))
	{
#ifdef CONFIG_CAN_DEBUG
		printk("%s %d Extended format error\n", __func__,__LINE__);
#endif
		result = -ENOSPC;
		goto Done;
	}
	else
	{
		if (regs->canrxfg.idr[1] & BUFFER_EXTENDED)
		{
			/* Extended ID. */
			frame->efi = MSGTYPE_EXTENDED;
			
			local.ucID[0] = regs->canrxfg.idr[0];
			local.ucID[1] = regs->canrxfg.idr[1];
			local.ucID[2] = regs->canrxfg.idr[4];
			local.ucID[3] = regs->canrxfg.idr[5];

			/* Shift data in 'local' * to remove the SRR and IDE bits. */
			local.dwID = (local.dwID & 0xFFE00000) |
                                     ((local.dwID & 0x0007FFFF) << 2 );			
			local.dwID >>= 3;
			frame->id = local.dwID;

			if (regs->canrxfg.idr[5] & BUFFER_EXT_RTR)
			{
				frame->rtr = MSGTYPE_RTR;
			}
		}
		else
		{
			/* Standart ID. */
			
			frame->efi  = MSGTYPE_STANDARD;

			local.ucID[0] = regs->canrxfg.idr[0];
			local.ucID[1] = regs->canrxfg.idr[1];

			local.dwID >>= 21;
			frame->id = local.dwID;

			if (regs->canrxfg.idr[1] & BUFFER_STD_RTR)
			{
				frame->rtr = MSGTYPE_RTR;
			}
		}
		
		do_gettimeofday(&frame->tv);
		/* Get data length.	 */
		ucLen = regs->canrxfg.dlr & 0x0F;
		frame->data_len = ucLen;
		
		/* Read data.	 */
		for (i = 0; i < ucLen; i++)
		{
			frame->data[i] = regs->canrxfg.dsr[i + (i / 2) * 2];
		}

	}
	
	result = 0;
Done:
	return result;
}

/* sent a CAN frame to a free slot */
static void mscan_write(struct can_mpc52xx_chip_t *chip, int bufslot, can_frame_t *frame)
{
	struct mpc52xx_mscan * regs    = chip->regs;
	volatile ID local;
	int i ;

	regs->cantier &= ~(1 << bufslot); 
	regs->cantbsel = 1 << bufslot;

	local.dwID = frame->id;

	if (frame->efi & MSGTYPE_EXTENDED)
	{
		local.dwID <<= 3;
		regs->cantxfg.idr[0]  = local.ucID[0];

		/* Write bits [20..18] of the ID. */
		regs->cantxfg.idr[1]  = local.ucID[1];

		/* set SRR and IDE bits	 */
		regs->cantxfg.idr[1] |= 0x18;

		/* Write [17..15] bits of the ID. */
		local.dwID >>= 2;
		regs->cantxfg.idr[1] &= ~0x07;
		regs->cantxfg.idr[1] |= (local.ucID[1] & 0x07);

		/* Now write the rest of the ID. */
		regs->cantxfg.idr[4]  = local.ucID[2];
		regs->cantxfg.idr[5]  = local.ucID[3];

		if (frame->rtr & MSGTYPE_RTR)
		{
			regs->cantxfg.idr[5] |= 0x1;
		}
		else
		{
			regs->cantxfg.idr[5] &= ~0x1;
		}
	}
	else
	{
		local.dwID   <<= 21;
		regs->cantxfg.idr[0]  = local.ucID[0];
		regs->cantxfg.idr[1]  = local.ucID[1];

		/* Clear IDE.	 */
		regs->cantxfg.idr[1] &= ~0x8;

		if (frame->rtr & MSGTYPE_RTR)
		{
			regs->cantxfg.idr[1] |= 0x10;
		}
		else
		{
			regs->cantxfg.idr[1] &= ~0x10;
		}
	}

	for (i = 0; i < frame->data_len; i++)
	{
		regs->cantxfg.dsr[i + (i / 2) * 2] = frame->data[i];
	}

	regs->cantxfg.dlr = frame->data_len;

	regs->cantxfg.tbpr = 0;	/* all messages have the same prio */

	/* Trigger transmission. */
	regs->cantflg = (1 << bufslot);
	
	/* Enable interrupt.*/
	regs->cantier |= (1 << bufslot);
	wmb();

#ifdef CONFIG_CAN_DEBUG
	printk(KERN_ERR "%s: mscan_write() OVER! \n", DEVICE_NAME );
#endif 
	return;
}

/* This function calculates BTR0 BTR1 values for a given bitrate.
 * Heavily based on mscan_bitrate(), which in turn is based
 * on work by Arnaud Westenberg.
 *
 * Set communication parameters.
 * baud rate in Hz
 * input clock frequency of MSCAN in Hz
 * sjw synchronization jump width (0-3) prescaled clock cycles
 * sampl_pt sample point in % (0-100) sets (TSEG1+2)/(TSEG1+TSEG2+3)
 *                                                               ratio
 */
static u16 mscan_bitrate(u32 rate)
{
	int best_error = 1000000000;
	int error;
	int best_tseg=0, best_brp=0, best_rate=0, brp=0;
	int tseg=0, tseg1=0, tseg2=0;

	/* Board info structure is not available to modules. */
	int clock = 66000000;
	u16 wBTR0BTR1;
	int sjw = 0;
	int sampl_pt = 90;
	int flags = 0;
	
	/* some heuristic specials */
	if (rate > ((1000000 + 500000) / 2))
		sampl_pt = 75;

	if (rate < ((12500 + 10000) / 2))
		sampl_pt = 75;
		
	if (rate < ((100000 + 125000) / 2))
		sjw = 1;

	/* tseg even = round down, odd = round up */
	for (tseg = (0 + 0 + 2) * 2;
	     tseg <= (MAX_TSEG2 + MAX_TSEG1 + 2) * 2 + 1;
	     tseg++)
	{
		brp = clock / ((1 + tseg / 2) * rate) + tseg % 2;
		if ((brp == 0) || (brp > 64))
			continue;
		
		error = rate - clock / (brp * (1 + tseg / 2));
		if (error < 0)
		{
			error = -error;
		}
			
		if (error <= best_error) 
		{
			best_error = error;
			best_tseg = tseg/2;
			best_brp = brp-1;
			best_rate = clock/(brp*(1+tseg/2));
		}
	}
	
	if (best_error && (rate / best_error < 10)) 
	{
		printk(KERN_ERR "%s: bitrate %d is not possible with %d Hz clock\n",
		        DEVICE_NAME, rate, clock);
		return 0;
	}
	
	tseg2 = best_tseg - (sampl_pt * (best_tseg + 1)) / 100;
	
	if (tseg2 < 0)
	{
		tseg2 = 0;
	}
		
	if (tseg2 > MAX_TSEG2)
	{
		tseg2 = MAX_TSEG2;
	}
		
	tseg1 = best_tseg - tseg2 - 2;
	
	if (tseg1 > MAX_TSEG1) 
	{
		tseg1 = MAX_TSEG1;
		tseg2 = best_tseg - tseg1 - 2;
	}

/* 
 * MSCAN has the sjw bits encoded not in the lexicographical
 * order, unlike the sja1000. We have to asjust for that here.
 */
	if (sjw && sjw < 3)
	{
		sjw ^= 3;/* 01 ==> 10  * 10 ==> 01	 */
	}

	wBTR0BTR1 = ((sjw << 6 | best_brp) << 8) |
	            (((flags & BTR1_SAM) != 0) << 7 | tseg2 << 4 | tseg1);

#ifdef CONFIG_CAN_DEBUG
	printk("bitrate %d : 0x%x\n", rate, wBTR0BTR1);
#endif

	return wBTR0BTR1;
}


/* Register a mpc52xx chip with the chip layer */
static int mscan_set_mode(struct can_mpc52xx_chip_t  * chip, int mode)
{
	struct mpc52xx_mscan * regs = chip->regs;
 	u8 canrier = regs->canrier;
	u8 cantier = regs->cantier;

#ifdef CONFIG_CAN_DEBUG
	printk(KERN_INFO "MSCAN: mscan_set_mode start ...\n");
#endif

	regs->canrier = 0;
	regs->cantier = 0;

	/* Switch to sleep mode */
	regs->canctl0 |= MPC52xx_MSCAN_SLPRQ;	
	while (!(regs->canctl1 & MPC52xx_MSCAN_SLPAK));
		
	/* Switch to initialization mode. */
	regs->canctl0 |= MPC52xx_MSCAN_INITRQ;
	while (!(regs->canctl1 & MPC52xx_MSCAN_INITAK));

	switch(mode){
		case MPC52xx_MSCAN_LISTEN:
			cantier = 0;
			regs->canctl1 |= MPC52xx_MSCAN_LISTEN;/* Set listen-only mode */
			break;
		case MPC52xx_MSCAN_LOOPB:
			regs->canctl1 |= MPC52xx_MSCAN_LOOPB;
			break;
		case MPC52xx_MSCAN_NORMAL:
			regs->canctl1 &= ~MPC52xx_MSCAN_LISTEN;
			regs->canctl1 &= ~MPC52xx_MSCAN_LOOPB;
			break;
		case MPC52xx_MSCAN_BITRATE:
			regs->canbtr0 = (chip->wBTR0BTR1 >> 8);
			regs->canbtr1 = (u8)chip->wBTR0BTR1;	
			break;
		default:			
			printk(KERN_ERR "MSCAN: mscan_set_mode Unknown mode = 0x%x.\n",mode);
	}	 

	/* Switch back to normal mode.	 */
	regs->canctl0 &= ~MPC52xx_MSCAN_INITRQ;
	regs->canctl0 &= ~MPC52xx_MSCAN_SLPRQ;
	while((regs->canctl1 & MPC52xx_MSCAN_INITAK) ||
	      (regs->canctl1 & MPC52xx_MSCAN_SLPAK));
	
	/* Enable receive interrupts.	 */
	regs->canrier = canrier;	
	regs->cantier = cantier;	
	
#ifdef CONFIG_CAN_DEBUG
	printk(KERN_INFO "MSCAN: mscan_set_mode end.\n");
#endif

	return 0;
}

/* -------------------chip function  end---------------------------------------*/


/* -------------------endpoint function start -------------------------------------- */
static int can_mpc52xx_initiate_read(struct can_endpoint_t *endpoint)
{
	struct can_mpc52xx_chip_t *chip = (struct can_mpc52xx_chip_t *) endpoint->chip;
	can_frame_t frame;
	
	if (endpoint->rxfifo.user_counter == 0) {
		/*Though there is no carer, so this frame should be throwed away */
#ifdef CONFIG_CAN_DEBUG		
		printk(KERN_WARNING "CAN: /dev/can%d no reader attached.\n",endpoint->minor);
#endif 		
		return -EAGAIN;
	} 

	mscan_read(chip,&frame);

	/* forward from frame to fifo.*/	
	return can_fifo_push(&endpoint->rxfifo,&frame);
}

/* Initiate a CAN endpoint write operation */
static int can_mpc52xx_initiate2write(struct can_endpoint_t *endpoint) {
	struct can_mpc52xx_chip_t *chip = (struct can_mpc52xx_chip_t *) endpoint->chip;
	struct mpc52xx_mscan * regs    = chip->regs;
	can_frame_t *frame = NULL;
	int   buf,used = 0;
	int retval = 0;
	
	can_fifo_lock(&endpoint->txfifo);
	/* Try to find an empty buffer and send a frame  */
	for (buf = 0; buf < 3; buf++)
	{
		if (!(regs->cantflg & (1 << buf))){
				continue;/* Buffer is busy. */
		}
		
		frame = can_fifo_pop(&endpoint->txfifo, 0, &retval);
			
		if (retval == 0) {
			mscan_write(endpoint->chip, buf, frame);
			used |= (1 << buf);
		}else{
#ifdef CONFIG_CAN_DEBUG			
			printk("No frame ready to be sent\n");
#endif
			break; /* There buffer is empty do nothing */
		}
	}
	
	can_fifo_unlock(&endpoint->txfifo);			
#ifdef CONFIG_CAN_DEBUG			
	printk("%s used = 0x%x t\n",__func__,used);
#endif
	return used;
}

/* Set the endpoint timing register */
static void can_mpc52xx_set_bitrate(struct can_endpoint_t *endpoint, int bitrate) 
{
	struct can_mpc52xx_chip_t *chip = (struct can_mpc52xx_chip_t *) endpoint->chip;
	u16 btr0btr1 = 0;

	btr0btr1 = mscan_bitrate(bitrate);

	chip->wBTR0BTR1 = btr0btr1;
	mscan_set_mode(chip,MPC52xx_MSCAN_BITRATE);	
	
	printk(KERN_INFO "MPC52xx: Setting bitrate %d btr0btr1=%x to can%d\n", bitrate,btr0btr1,endpoint->minor);

	return;
}

/* set the chip to mode */
static void can_mpc52xx_set_mode(struct can_endpoint_t *endpoint, int mode)
{
	struct can_mpc52xx_chip_t  * chip = endpoint->chip;

	switch (mode) 	{
	case MPC52xx_MSCAN_LISTEN:
		chip->loopbackmode = 0;
		chip->ucListenOnly = 1;		
		break;
	case MPC52xx_MSCAN_LOOPB:
		chip->loopbackmode = 1;
		chip->ucListenOnly = 0;
		break;
	case MPC52xx_MSCAN_NORMAL:
	default:	
		chip->loopbackmode = 0;
		chip->ucListenOnly = 0;
	}
	
	mscan_set_mode(chip,mode);
	
	printk(KERN_INFO "MPC52xx: set_mode %s to CAN%d \n", 
		(mode == MPC52xx_MSCAN_LOOPB ? "LOOPBACK" :  
		(mode == MPC52xx_MSCAN_LISTEN ? "ListenOnly" : 
		(mode == MPC52xx_MSCAN_NORMAL ? "Normal": "Unkown" ) )),endpoint->minor);

	return;
}

/*chip resest function */
static void can_mpc52xx_reset(struct can_endpoint_t *endpoint)
{
	struct can_mpc52xx_chip_t  * chip = endpoint->chip;
	struct mpc52xx_mscan * regs = chip->regs;

#ifdef CONFIG_CAN_DEBUG
	printk(KERN_INFO "MSCAN: can_mpc52xx_reset start ...\n");
#endif

	regs->canrier = 0;
	regs->cantier = 0;
	chip->loopbackmode = 0;
	chip->ucListenOnly = 0;	
	mscan_init(chip);
	
#ifdef CONFIG_CAN_DEBUG
	printk(KERN_INFO "MSCAN: can_mpc52xx_reset end.\n");
#endif	
	return;
}

/*  Register a mpc52xx chip and provide an interface to the can-base layer  */
static int can_mpc52xx_register(struct can_mpc52xx_chip_t *chip) 
{
	int retval = 0;
	struct can_endpoint_t *endpoint = chip->endpoint;

	retval = can_endpoint_register(endpoint);

	if(retval < 0) {
		printk(KERN_ERR "CAN: wicked error from out of device endpoint");
		return retval;
	}

	endpoint->chip = chip;
	endpoint->initiate2write = can_mpc52xx_initiate2write;
	endpoint->set_bitrate = can_mpc52xx_set_bitrate;	
	endpoint->chip_reset = can_mpc52xx_reset;
	endpoint->set_mode = can_mpc52xx_set_mode;

	return retval;
}
/* -------------------endpoint function end -------------------------------------- */


/* -------------------OF function start -------------------------------------- */
/*This function may be re-entry, so don't use static melta. */
static irqreturn_t can_mpc52xx_interrupter(int irq, void *dev_id, struct pt_regs *pt_regs)
{
	struct can_mpc52xx_chip_t  * chip = (struct can_mpc52xx_chip_t *)dev_id;
	struct can_endpoint_t *endpoint = chip->endpoint;	
	struct mpc52xx_mscan * regs  = chip->regs;
	unsigned long flags;
	int  err;
	int  wr_wakeup   = 0;
	int  rd_wakeup   = 0;
	u8  t_status, r_status,used = 0;

	spin_lock_irqsave(&chip->lock, flags);
	t_status = regs->cantflg;
	r_status = regs->canrflg;

#ifdef CONFIG_CAN_DEBUG
	printk(KERN_ERR "%s: can_mpc52xx_interrupter() r = 0x%x t = 0x%x\n",
	        DEVICE_NAME, r_status, t_status);
#endif 
	if (r_status & MPC52xx_MSCAN_RXF)
	{
		/* Handle receiption. */
		if ((err = can_mpc52xx_initiate_read(endpoint)))
		{
			chip->error_counter++;
			chip->error_status |= CAN_ERR_QOVERRUN;
		}
		rd_wakeup = 1;
	}

	if (t_status & MPC52xx_MSCAN_TXE)
	{
		/* reading doc to be sure that it is needed */
		regs->cantier &= ~(MPC52xx_MSCAN_TXIE & t_status ); 
		used = can_mpc52xx_initiate2write(endpoint);
		wr_wakeup = 1;
		/* Signal to:  there is already idle sent slot.	*/
		if((t_status & MPC52xx_MSCAN_TXE) & (~(used) ))
			atomic_set(&endpoint->send_ready_flag, 1);

	}

	if (r_status & MPC52xx_MSCAN_OVRIF)
	{
		/* Handle data overrun.	 */
		chip->error_status |= CAN_ERR_QOVERRUN;
		rd_wakeup = 1;
		chip->error_counter++;
	}

	if (wr_wakeup)
	{
		wake_up_interruptible(&endpoint->write_queue);
	}

	if (rd_wakeup)
	{
		wake_up_interruptible(&endpoint->read_queue);
	}
	/* Ack receive interrupter */
	regs->canrflg = r_status;
	spin_unlock_irqrestore(&chip->lock, flags);
	return IRQ_HANDLED;
}

static int __devinit can_mpc52xx_probe(struct of_device *of_dev, const struct of_device_id *match)
{
	struct resource res ;
	struct device_node *dp = of_dev->node;
	struct can_mpc52xx_chip_t *chip = NULL;
	void __iomem *base = NULL;	
	int irq,err = -1,minor;

	if (of_address_to_resource(dp, 0, &res)) {
		dev_err(&of_dev->dev, "Can't get the can mapping!\n");
		err = -EINVAL;
		goto err_out;
	}

#ifdef CONFIG_CAN_DEBUG
      	printk("mpc52xx can device: %.8llx at %.8llx\n",
	    (unsigned long long)res.end - res.start + 1,
	    (unsigned long long)res.start);
#endif 

	irq = irq_of_parse_and_map(dp, 0);

	if (irq < 0) {
		err = -ENXIO;
		goto err_out;
	}

	base = ioremap((phys_addr_t)res.start, (res.end - res.start + 1));

	if (!base) {
		printk(KERN_ERR "CAN-mpc52xx - failed to map controller\n");
		err = -ENXIO;
		goto err_out;
	}

	chip = (struct can_mpc52xx_chip_t *)kmalloc(sizeof(struct can_mpc52xx_chip_t), GFP_KERNEL);	

	if(chip == NULL) {
		err = -ENOMEM;
		goto ioerr_out;
	}

	chip->irq = irq;
	chip->regs = (struct mpc52xx_mscan *)base;
	chip->bitrate = 1000000;
	chip->wBTR0BTR1 = mscan_bitrate(chip->bitrate);
	chip->error_status   = 0;
	chip->bExtended    = 1;
	chip->ucListenOnly = 0;
	chip->loopbackmode = 0;
	spin_lock_init(&chip->lock);
	dev_set_drvdata(&of_dev->dev, chip);	
	
	/* Try to get can endpoint pointer, here 2 is a speed ratio of Tx/Rx*/
	chip->endpoint = can_endpoint_init(2);
	if(chip->endpoint == NULL) {
		err = -ENOMEM;		
		goto free_chip_out ;
	}
	
	minor = can_mpc52xx_register(chip);
	if(minor < 0 ) {
		can_endpoint_free(chip->endpoint);
		goto free_chip_out;	
	}

	if ((err = request_irq(irq, (irq_handler_t)can_mpc52xx_interrupter,  IRQF_SHARED, "can-mpc52xx", chip)) < 0) {
			printk(KERN_ERR "can-mpc52xx - failed to attach interrupt\n");
			err = -ENXIO;
			goto free_chip_out;
	}	
	
	mscan_init(chip);	
	/* There is no ongoing send */
	atomic_set(&chip->endpoint->send_ready_flag, 1);	
		
	printk(KERN_INFO "CAN:Irq %d BaudR %d %s %s on /dev/can%d.\n",
		irq,chip->bitrate,(chip->bExtended == 1)? "Ext.":"Std.", 
			(chip->loopbackmode ==1)? "LB" : "Normal", minor);
	return 0;

free_chip_out:
	kfree(chip);	
ioerr_out:
	iounmap(base);
err_out:	
	return err;
}

static int can_mpc52xx_remove(struct of_device *of_dev)
{
	struct can_mpc52xx_chip_t *chip = (struct can_mpc52xx_chip_t *)dev_get_drvdata(&of_dev->dev);

	can_endpoint_unregister(chip->endpoint);
	dev_set_drvdata(&of_dev->dev, NULL);
	if (chip->irq != 0)
		free_irq(chip->irq, chip);

	iounmap(chip->regs);
	kfree(chip);

	return 0;
}


static struct of_device_id can_mpc52xx_match[] = {
	{
		.type		= "mscan",
		.compatible	= "mpc5200b-mscan"
	},
	{ },
};

MODULE_DEVICE_TABLE(of, can_mpc52xx_match);

static struct of_platform_driver can_mpc52xx_driver = {
	.name		= "mpc52xx-mscan",
	.match_table	= can_mpc52xx_match,
	.probe		= can_mpc52xx_probe,
	.remove		= can_mpc52xx_remove,
};

static int __init mpc52xx_can_init_module(void)
{
	return of_register_platform_driver(&can_mpc52xx_driver);
}

static void __exit mpc52xx_can_cleanup_module(void)
{
	of_unregister_platform_driver(&can_mpc52xx_driver);
}

/* -------------------OF function end -------------------------------------- */

module_init(mpc52xx_can_init_module);
module_exit(mpc52xx_can_cleanup_module);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("MPC52xx can driver for OF");
