/*
 * MSCAN defines
 * from mpc52xx.h
 *
 *  Maintainer : Sylvain Munaut <tnt@246tNt.com>
 *
 *  Copyright (C) 2004 Sylvain Munaut <tnt@246tNt.com>
 *  Copyright (C) 2003 MontaVista, Software, Inc.
 *
 * This file is licensed under the terms of the GNU General Public License
 * version 2. This program is licensed "as is" without any warranty of any
 * kind, whether express or implied.
 */

#ifndef __MPC52xx_MSCAN_H__
#define __MPC52xx_MSCAN_H__

#define DEVICE_NAME "mpc52xx-mscan"

/* MSCAN control register 0 (CANCTL0) bits */
#define MPC52xx_MSCAN_RXFRM	0x80
#define MPC52xx_MSCAN_RXACT	0x40
#define MPC52xx_MSCAN_CSWAI	0x20
#define MPC52xx_MSCAN_SYNCH	0x10
#define MPC52xx_MSCAN_TIME	0x08
#define MPC52xx_MSCAN_WUPE	0x04
#define MPC52xx_MSCAN_SLPRQ	0x02
#define MPC52xx_MSCAN_INITRQ	0x01

/* MSCAN control register 1 (CANCTL1) bits */
#define MPC52xx_MSCAN_CANE	0x80
#define MPC52xx_MSCAN_CLKSRC	0x40
#define MPC52xx_MSCAN_LOOPB	0x20
#define MPC52xx_MSCAN_LISTEN	0x10
#define MPC52xx_MSCAN_WUPM	0x04
#define MPC52xx_MSCAN_SLPAK	0x02
#define MPC52xx_MSCAN_INITAK	0x01

/* MSCAN receiver flag register (CANRFLG) bits */
#define MPC52xx_MSCAN_WUPIF	0x80
#define MPC52xx_MSCAN_CSCIF	0x40
#define MPC52xx_MSCAN_RSTAT1	0x20
#define MPC52xx_MSCAN_RSTAT0	0x10
#define MPC52xx_MSCAN_TSTAT1	0x08
#define MPC52xx_MSCAN_TSTAT0	0x04
#define MPC52xx_MSCAN_OVRIF	0x02
#define MPC52xx_MSCAN_RXF		0x01

/* MSCAN receiver interrupt enable register (CANRIER) bits */
#define MPC52xx_MSCAN_WUPIE	0x80
#define MPC52xx_MSCAN_CSCIE	0x40
#define MPC52xx_MSCAN_RSTATE1	0x20
#define MPC52xx_MSCAN_RSTATE0	0x10
#define MPC52xx_MSCAN_TSTATE1	0x08
#define MPC52xx_MSCAN_TSTATE0	0x04
#define MPC52xx_MSCAN_OVRIE	0x02
#define MPC52xx_MSCAN_RXFIE	0x01

/* MSCAN transmitter flag register (CANTFLG) bits */
#define MPC52xx_MSCAN_TXE2	0x04
#define MPC52xx_MSCAN_TXE1	0x02
#define MPC52xx_MSCAN_TXE0	0x01
#define MPC52xx_MSCAN_TXE	(MPC52xx_MSCAN_TXE2 | MPC52xx_MSCAN_TXE1 | \
				 MPC52xx_MSCAN_TXE0)

/* MSCAN transmitter interrupt enable register (CANTIER) bits */
#define MPC52xx_MSCAN_TXIE2	0x04
#define MPC52xx_MSCAN_TXIE1	0x02
#define MPC52xx_MSCAN_TXIE0	0x01
#define MPC52xx_MSCAN_TXIE	(MPC52xx_MSCAN_TXIE2 | MPC52xx_MSCAN_TXIE1 | \
				 MPC52xx_MSCAN_TXIE0)

/* MSCAN transmitter message abort request (CANTARQ) bits */
#define MPC52xx_MSCAN_ABTRQ2	0x04
#define MPC52xx_MSCAN_ABTRQ1	0x02
#define MPC52xx_MSCAN_ABTRQ0	0x01

/* MSCAN transmitter message abort ack (CANTAAK) bits */
#define MPC52xx_MSCAN_ABTAK2	0x04
#define MPC52xx_MSCAN_ABTAK1	0x02
#define MPC52xx_MSCAN_ABTAK0	0x01

/* MSCAN transmit buffer selection (CANTBSEL) bits */
#define MPC52xx_MSCAN_TX2	0x04
#define MPC52xx_MSCAN_TX1	0x02
#define MPC52xx_MSCAN_TX0	0x01

/* MSCAN ID acceptance control register (CANIDAC) bits */
#define MPC52xx_MSCAN_IDAM1	0x20
#define MPC52xx_MSCAN_IDAM0	0x10
#define MPC52xx_MSCAN_IDHIT2	0x04
#define MPC52xx_MSCAN_IDHIT1	0x02
#define MPC52xx_MSCAN_IDHIT0	0x01

#define MPC52xx_MSCAN_NORMAL	0x00
#define MPC52xx_MSCAN_BITRATE	0x01

struct mscan_buffer {
	volatile u8  idr[0x8];		/* 0x00 */
	volatile u8  dsr[0x10];		/* 0x08 */
	volatile u8  dlr;		/* 0x18 */
	volatile u8  tbpr;		/* 0x19 */	/* This register is not applicable for receive buffers */
	volatile u16 rsrv1;		/* 0x1A */
	volatile u8  tsrh;		/* 0x1C */
	volatile u8  tsrl;		/* 0x1D */
	volatile u16 rsrv2;		/* 0x1E */
};

struct mpc52xx_mscan {
	volatile u8  canctl0;		/* MSCAN + 0x00 */
	volatile u8  canctl1;		/* MSCAN + 0x01 */
	volatile u16 rsrv1;		/* MSCAN + 0x02 */
	volatile u8  canbtr0;		/* MSCAN + 0x04 */
	volatile u8  canbtr1;		/* MSCAN + 0x05 */
	volatile u16 rsrv2;		/* MSCAN + 0x06 */
	volatile u8  canrflg;		/* MSCAN + 0x08 */
	volatile u8  canrier;		/* MSCAN + 0x09 */
	volatile u16 rsrv3;		/* MSCAN + 0x0A */
	volatile u8  cantflg;		/* MSCAN + 0x0C */
	volatile u8  cantier;		/* MSCAN + 0x0D */
	volatile u16 rsrv4;		/* MSCAN + 0x0E */
	volatile u8  cantarq;		/* MSCAN + 0x10 */
	volatile u8  cantaak;		/* MSCAN + 0x11 */
	volatile u16 rsrv5;		/* MSCAN + 0x12 */
	volatile u8  cantbsel;		/* MSCAN + 0x14 */
	volatile u8  canidac;		/* MSCAN + 0x15 */
	volatile u16 rsrv6[3];		/* MSCAN + 0x16 */
	volatile u8  canrxerr;		/* MSCAN + 0x1C */
	volatile u8  cantxerr;		/* MSCAN + 0x1D */
	volatile u16 rsrv7;		/* MSCAN + 0x1E */
	volatile u8  canidar0;		/* MSCAN + 0x20 */
	volatile u8  canidar1;		/* MSCAN + 0x21 */
	volatile u16 rsrv8;		/* MSCAN + 0x22 */
	volatile u8  canidar2;		/* MSCAN + 0x24 */
	volatile u8  canidar3;		/* MSCAN + 0x25 */
	volatile u16 rsrv9;		/* MSCAN + 0x26 */
	volatile u8  canidmr0;		/* MSCAN + 0x28 */
	volatile u8  canidmr1;		/* MSCAN + 0x29 */
	volatile u16 rsrv10;		/* MSCAN + 0x2A */
	volatile u8  canidmr2;		/* MSCAN + 0x2C */
	volatile u8  canidmr3;		/* MSCAN + 0x2D */
	volatile u16 rsrv11;		/* MSCAN + 0x2E */
	volatile u8  canidar4;		/* MSCAN + 0x30 */
	volatile u8  canidar5;		/* MSCAN + 0x31 */
	volatile u16 rsrv12;		/* MSCAN + 0x32 */
	volatile u8  canidar6;		/* MSCAN + 0x34 */
	volatile u8  canidar7;		/* MSCAN + 0x35 */
	volatile u16 rsrv13;		/* MSCAN + 0x36 */
	volatile u8  canidmr4;		/* MSCAN + 0x38 */
	volatile u8  canidmr5;		/* MSCAN + 0x39 */
	volatile u16 rsrv14;		/* MSCAN + 0x3A */
	volatile u8  canidmr6;		/* MSCAN + 0x3C */
	volatile u8  canidmr7;		/* MSCAN + 0x3D */
	volatile u16 rsrv15;		/* MSCAN + 0x3E */
	
	struct mscan_buffer canrxfg;	/* MSCAN + 0x40 */    /* Foreground receive buffer */
	struct mscan_buffer cantxfg;	/* MSCAN + 0x60 */    /* Foreground transmit buffer */
};

/* -------data structure definition start-----------*/

/* Available device types */
enum device_type_t {
	DT_SINGLE,           /* Single-endpoint device */
	DT_SINGLE_NOEXTCLK,  /* Single-endpoint device, without external clock source */
	DT_MASTER,           /* Dual-endpoint: master device */
	DT_SLAVE             /* Dual-endpoint: slave device */
};

typedef union
{
	u8  ucID[4];
	u32 dwID;
} ID;

/* Fundamental mpc52xx chip operations */
struct can_mpc52xx_chip_t {
	/* Pointer to the chip's CAN endpoint */
	struct can_endpoint_t *endpoint;
	/* Pointer to the chip's hw device */
	struct mpc52xx_mscan *regs;
	/* Device type */
	enum device_type_t type;
	int irq;
	
	spinlock_t lock;		/* No recursive interrupts */
 	int  bitrate; 		
	u32  error_counter;		/* counts all fatal errors*/	
 	u32	reserved0;	
	u32  reserved1;		
	u16  error_status;		/* status of CAN chip*/
	u16  wBTR0BTR1;		/*  Bitrate BTR0 and BTR1*/
	u8   ucCANMsgType;	/* the persistent storage for 11 or 29 bit identifier*/
	u8   ucListenOnly;	/* the persistent storage for listen-only mode*/
	u8   loopbackmode;	 /* loopback mode  */
	u8   bExtended;		/* if 0, no extended frames are accepted*/		
};

/* -------data structure definition start-----------*/

/*  static declaration start */

/* Max. number of messages during an interrupt */
#define CAN_MPC52xx_MAX_MESSAGES		8

/* Max. number of interrupts to process before leaving the handler */
#define CAN_MPC52xx_MAX_INTERRUPTS	8
#define MAX_TSEG1	15
#define MAX_TSEG2	7
#define BTR1_SAM	(1 << 1)
 
/* Message type access macros. */
#define BUFFER_STD_RTR		0x10
#define BUFFER_EXT_RTR		0x01
#define BUFFER_EXTENDED		0x08

#define CAN_BAUD_1M	0x025e	/*    1 MBit/s        */
#define CAN_BAUD_500K	0x054f	 /*   500 kBit/s    */
#define CAN_BAUD_250K	0x0a6f	/*   250 kBit/s    */
#define CAN_BAUD_125K	0x156f 	/*   125 kBit/s    */
#define CAN_BAUD_100K	0x9d4f	 /*   100 kBit/s    */
#define CAN_BAUD_50K	0xb66f	/*   50 kBit/s      */
#define CAN_BAUD_20K	0x0000	/*   not supported*/
#define CAN_BAUD_10K	0x0000	/*   not supported*/
#define CAN_BAUD_5K	0x0000	/*   not supported*/

/* MSGTYPE bits of element MSGTYPE in structure TPCANMsg*/
#define MSGTYPE_STATUS		0x80	/* used to mark pending status */
#define MSGTYPE_EXTENDED	0x03	/* declares a extended frame */
#define MSGTYPE_RTR			0x01	/* marks a remote frame*/
#define MSGTYPE_STANDARD	0x00	/* marks a standard frame*/

/* error codes */
#define CAN_ERR_QOVERRUN	0x0040	/*  receive queue overrun*/

#endif /* __MPC52xx_MSCAN_H__ */
