/*
 * drivers/ide/ppc/mpc52xx_ide_iops.c
 *
 * Utility functions for MPC52xx on-chip IDE interface
 *
 *
 * Copyright (C) 2006 Sylvain Munaut <tnt@246tNt.com>
 * Copyright (C) 2003 Mipsys - Benjamin Herrenschmidt
 *
 * This file is licensed under the terms of the GNU General Public License
 * version 2. This program is licensed "as is" without any warranty of any
 * kind, whether express or implied.
 */

#include <linux/kernel.h>
#include <linux/ide.h>
#include <asm/io.h>

#include "mpc52xx_ide.h"



static u8
mpc52xx_ide_inb(unsigned long port)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	return (u8) readb((void __iomem *) port);
}

static u16
mpc52xx_ide_inw(unsigned long port)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	return (u16) readw((void __iomem *) port);
}

static void
mpc52xx_ide_insw(unsigned long port, void *addr, u32 count)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	__ide_mm_insw((void __iomem *) port, addr, count);
}

static u32
mpc52xx_ide_inl(unsigned long port)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	return (u32) readl((void __iomem *) port);
}

static void
mpc52xx_ide_insl(unsigned long port, void *addr, u32 count)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	__ide_mm_insl((void __iomem *) port, addr, count);
}

static void
mpc52xx_ide_outb(u8 value, unsigned long port)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	writeb(value, (void __iomem *) port);
}

static void
mpc52xx_ide_outbsync(ide_drive_t *drive, u8 value, unsigned long port)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	writeb(value, (void __iomem *) port);
}


static void
mpc52xx_ide_outw(u16 value, unsigned long port)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	writew(value, (void __iomem *) port);
}

static void
mpc52xx_ide_outsw(unsigned long port, void *addr, u32 count)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	__ide_mm_outsw((void __iomem *) port, addr, count);
}

static void
mpc52xx_ide_outl(u32 value, unsigned long port)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	writel(value, (void __iomem *) port);
}

static void
mpc52xx_ide_outsl(unsigned long port, void *addr, u32 count)
{
	struct mpc52xx_ata __iomem *ata_regs =
		(struct mpc52xx_ata __iomem *)(port & ~0xfful);

	mpc52xx_ide_wait_tip_bit_clear(ata_regs);
	__ide_mm_outsl((void __iomem *) port, addr, count);
}


void
mpc52xx_ide_setup_hwif_iops(ide_hwif_t *hwif)
{
	hwif->OUTB	= mpc52xx_ide_outb;
	hwif->OUTBSYNC	= mpc52xx_ide_outbsync;
	hwif->OUTW	= mpc52xx_ide_outw;
/*	hwif->OUTL	= mpc52xx_ide_outl; */
	hwif->OUTSW	= mpc52xx_ide_outsw;
	hwif->OUTSL	= mpc52xx_ide_outsl;
	hwif->INB	= mpc52xx_ide_inb;
	hwif->INW	= mpc52xx_ide_inw;
/*	hwif->INL	= mpc52xx_ide_inl; */
	hwif->INSW	= mpc52xx_ide_insw;
	hwif->INSL	= mpc52xx_ide_insl;
}

