/*
 * drivers/mtd/maps/pramc6201_bootflash .c
 * 
 * mapping driver for boot flash device (s29GL256N 1x265Mb)
 *
 * Author: Majid Khan M Z <amm117@motorola.com>
 *
 * Copyright 
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <asm/io.h>
#include <platforms/pamc6201.h>

#define PAMC6201_BOOT_FLASH_NAME         "PrAMC-6201 boot flash"

static struct mtd_info *bootbank;

static struct map_info pramc6201_bootbank_map = {
	.name		= PAMC6201_BOOT_FLASH_NAME,
	.size		= PAMC6201_BOOT_FLASH_SIZE_ACTUAL,
	.bankwidth	= 2,
};

static struct mtd_partition pramc6201_bootbank_part[] = {
	{
		.name		= "primary boot sector (Read-only)",
		.size		= 0xf00000,
		.offset		= 0x0,
		.mask_flags	= MTD_WRITEABLE, 	/* forced to read-only */
    	},
	{
        	.name		= "primary boot sector",
		.size		= 0x0100000,
		.offset		= 0xf00000,
	},
};

int __init init_pramc6201_bootbank(void)
{
	unsigned long long bootbank_flash_base;

	bootbank_flash_base = PAMC6201_BOOT_FLASH_BASE;

	pramc6201_bootbank_map.phys = bootbank_flash_base;
	pramc6201_bootbank_map.virt = ioremap_nocache(bootbank_flash_base,
                                                           pramc6201_bootbank_map.size);
 
	if (!pramc6201_bootbank_map.virt) {
        	printk("Failed to ioremap Boot Bank Flash\n");
		return -EIO;
	}

	simple_map_init(&pramc6201_bootbank_map);

	bootbank = do_map_probe("cfi_probe", &pramc6201_bootbank_map);
	if (bootbank) {
        	bootbank->owner = THIS_MODULE;
		add_mtd_partitions(bootbank, pramc6201_bootbank_part,
                		ARRAY_SIZE(pramc6201_bootbank_part));
	} else {
        	printk("Map Probe Failed for Boot Bank Flash\n");
		if(pramc6201_bootbank_map.virt)
	        	iounmap((void *)pramc6201_bootbank_map.virt);
		return -ENXIO;
	}

	return 0;
}

static void __exit cleanup_pramc6201_bootbank(void)
{
	if (bootbank) {
		del_mtd_partitions(bootbank);
		map_destroy(bootbank);
	}

	if (pramc6201_bootbank_map.virt) {
		iounmap((void *)pramc6201_bootbank_map.virt);
		pramc6201_bootbank_map.virt = 0;
	}
}

module_init(init_pramc6201_bootbank);
module_exit(cleanup_pramc6201_bootbank);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Majid Khan M Z <amm117@motorola.com>");
MODULE_DESCRIPTION("mapping driver for Motorola PrAMC-6201 boot flash device");

