/*
 * linux/include/asm-arm/arch-omap/board-2430sdp.h
 *
 * Hardware definitions for TI OMAP2430 SDP board.
 *
 * Based on board-h4.h by Dirk Behme <dirk.behme@de.bosch.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __ASM_ARCH_OMAP_2430SDP_H
#define __ASM_ARCH_OMAP_2430SDP_H

/* Placeholder for 2430SDP specific defines */
#define OMAP24XX_ETHR_START		 0x08000300
#define OMAP24XX_ETHR_GPIO_IRQ		149
#define SDP2430_CS0_BASE		0x04000000

#define TWL4030_IRQNUM			INT_24XX_SYS_NIRQ

/* TWL4030 Primary Interrupt Handler (PIH) interrupts */
#define IH_TWL4030_BASE			IH_BOARD_BASE
#define IH_TWL4030_END			(IH_TWL4030_BASE+8)
#define NR_IRQS				(IH_TWL4030_END)

/* NAND */
/* IMPORTANT NOTE ON MAPPING
 * H4-242X
 * -------
 * CS0 - Either NOR or NAND - always mapped to 0x04000000
 * CS1 - MPDB - debug board
 *
 * 2430SDP - 243X
 * ----------
 * NOR always on  0x04000000
 * MPDB always on 0x08000000
 * NAND always on 0x0C000000
 * OneNand Mapped to 0x20000000
 * Sibley Mapped to 0x10000000 and 0x14000000 (adjacent address space) 0x1..->0x1ffff.. is reserved for sibley
 * NOR or NAND on CS0 based on Boot Mode (NAND/NOR)
 * The other on CS1
 */
# define FLASH_BASE                     0x04000000  /* NOR flash (64 Meg aligned) */
# define DEBUG_BASE                     0x08000000  /* debug board */
# define NAND_BASE                      0x0C000000  /* NAND flash */
# define SIBLEY_MAP1                    0x10000000  /* Sibley1 flash */
# define SIBLEY_MAP2                    0x14000000  /* Sibley2 flash */
# define ONENAND_MAP                    0x20000000  /* OneNand flash */

#define OMAP_NAND_FLASH_START1           NAND_BASE

#define OMAP_NAND_FLASH_SIZE             SZ_64M
#define OMAP_NAND_FLASH_START2           NAND_BASE

#define OMAP_DIPSWITCH_REG		(DEBUG_BASE + 0x62)
#define OMAP_DIPSWITCH_S10_4_MASK	0x800

#endif /* __ASM_ARCH_OMAP_2430SDP_H */
