/*
 * linux/include/asm-arm/arch-omap2/bus.h
 *
 * Virtual bus for OMAP2. Allows better power management, such as managing
 * shared clocks, and mapping of bus addresses to Local Bus addresses.
 *
 * Copyright (C) 2004 Texas Instruments, Inc.
 *
 * This package is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef __ASM_ARM_ARCH_OMAP2_BUS_H
#define __ASM_ARM_ARCH_OMAP2_BUS_H

extern struct bus_type omap_bus_types[];

/*
 * Description for physical device
 */
struct omap_dev {
	struct device dev;	/* Standard device description */
	char *name;
	unsigned int devid;	/* OMAP device id */
	unsigned int busid;	/* OMAP virtual busid */
	struct resource res;	/* Standard resource description */
	void *mapbase;		/* OMAP physical address */
	unsigned int irq[6];	/* OMAP interrupts */
};

#define OMAP_DEV(_d)	container_of((_d), struct omap_dev, dev)

#define omap_get_drvdata(d)	dev_get_drvdata(&(d)->dev)
#define omap_set_drvdata(d,p)	dev_set_drvdata(&(d)->dev, p)

#define OMAP24xx_CAM_DEVID    0
#define OMAP24xx_SERIAL_DEVID 1
#define OMAP24xx_MMC1_DEVID    2
#define OMAP24xx_I2C_DEVID    3
#define OMAP24xx_ETH_DEVID    4
#define OMAP24xx_WDT_DEVID    5
#define OMAP24xx_FB_DEVID     6
#define OMAP24xx_AUDIO_DEVID  7
#define OMAP24xx_TS_DEVID     8
#define OMAP24xx_LCD_DEVID    9
#define OMAP24xx_TV_DEVID     10
#define OMAP24xx_V1OUT_DEVID  11
#define OMAP24xx_V2OUT_DEVID  12
#define OMAP24xx_TRITON_DEVID 13
#define OMAP24xx_KP_DEVID     14
#define OMAP24xx_IR_DEVID     15
#define OMAP24xx_RTC_DEVID    16
#define OMAP24xx_MMC2_DEVID   17 
#define OMAP24xx_DSP_DEVID    18 
#define OMAP24xx_BATT_DEVID   19
#define OMAP24xx_OSTIMER_DEVID 20

/*
 * Description for device driver
 * - Note the usage from 2.6.10 -> 2.6.14 -> 2.6.19 has chagned for 'state'
 *   the generic driver alwasy gets a pm_message_t, this goes to the bus driver.
 *   the bus drivers handling of this has changed from kernel to kernel.  OMAP
 *   bus drivers are currently sticking with a u32 state message.  The potentially
 *   variable pm_message_t state is not currently passed to clients.
 */
struct omap_driver {
	struct device_driver drv;	/* Standard driver description */
	unsigned int devid;	/* OMAP device id for bus */
	unsigned int busid;	/* OMAP virtual busid */
	unsigned int clocks;	/* OMAP shared clocks */
	int (*probe) (struct omap_dev *);
	int (*remove) (struct omap_dev *);
	int (*suspend) (struct omap_dev *, u32 state);
	int (*resume) (struct omap_dev *);
};

#define OMAP_DRV(_d)	container_of((_d), struct omap_driver, drv)
#define OMAP_DRIVER_NAME(_omapdev) ((_omapdev)->dev.driver->name)

/*
 * Device ID numbers for bus types
 */

/*
 * Virtual bus definitions for OMAP
 */
#ifdef CONFIG_ARCH_OMAP242X
#define OMAP_NR_BUSES	2

#define OMAP_BUS_NAME_L3	"L3"
#define OMAP_BUS_NAME_L4	"L4"
enum {
	OMAP_BUS_L3 = 0,
	OMAP_BUS_L4,
};
#elif CONFIG_ARCH_OMAP2430
#define OMAP_NR_BUSES	3

#define OMAP_BUS_NAME_L3	"L3"
#define OMAP_BUS_NAME_L4	"L4CORE"
#define OMAP_BUS_NAME_L4_WKUP	"L4WKUP"
enum {
	OMAP_BUS_L3 = 0,
	OMAP_BUS_L4_WKUP,
	OMAP_BUS_L4,
};
#endif


/* See arch/arm/mach-omap2/bus.c for the rest of the bus definitions. */

extern int omap_driver_register(struct omap_driver *driver);
extern void omap_driver_unregister(struct omap_driver *driver);
extern int omap_device_register(struct omap_dev *odev);
extern void omap_device_unregister(struct omap_dev *odev);

#endif				/* __ASM_ARM_ARCH_OMAP2_BUS_H */
