/*
 * linux/include/asm-arm/arch-omap/dma.h
 *
 * Defines for OMAP2 DMA
 *
 * Copyright (C) 2003 Nokia Corporation
 * Author: Juha Yrjölä <juha.yrjola@nokia.com>
 *
 * Copyright (C) 2004 - 2006 Texas Instruments, Inc.
 *
 * This package is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef __ASM_ARM_ARCH_OMAP2_DMA_H
#define __ASM_ARM_ARCH_OMAP2_DMA_H

/* require base address of L4 */
#include <asm/arch/hardware.h>

#define MAX_DMA_ADDRESS                 0xffffffff
#define MAX_DMA_CHANNELS                0

#define OMAP_DMA_TOUT_IRQ               (1 << 0)
#define OMAP_DMA_DROP_IRQ               (1 << 1)
#define OMAP_DMA_HALF_IRQ               (1 << 2)
#define OMAP_DMA_FRAME_IRQ              (1 << 3)
#define OMAP_DMA_LAST_IRQ               (1 << 4)
#define OMAP_DMA_BLOCK_IRQ              (1 << 5)
#define OMAP_DMA_SYNC_IRQ               (1 << 6)
#define OMAP_DMA_TRANSERR_IRQ           (1 << 8)
#define OMAP_DMA_MISALIGN_IRQ           (1 << 11)
#define OMAP_DMA_SUPERVISOR_IR          (1 << 10)

#define OMAP_DMA_DATA_TYPE_S8           0x00
#define OMAP_DMA_DATA_TYPE_S16          0x01
#define OMAP_DMA_DATA_TYPE_S32          0x02

#define OMAP_DMA_SYNC_ELEMENT           0x00
#define OMAP_DMA_SYNC_FRAME             0x01
#define OMAP_DMA_SYNC_BLOCK             0x02

#define OMAP_DMA_SRC_SYNC		0x01
#define OMAP_DMA_DST_SYNC		0x00

#define OMAP_DMA_PORT_EMIFF             0x00
#define OMAP_DMA_PORT_EMIFS             0x01
#define OMAP_DMA_PORT_OCP_T1            0x02
#define OMAP_DMA_PORT_TIPB              0x03
#define OMAP_DMA_PORT_OCP_T2            0x04
#define OMAP_DMA_PORT_MPUI              0x05

#define OMAP_DMA_AMODE_CONSTANT         0x00
#define OMAP_DMA_AMODE_POST_INC         0x01
#define OMAP_DMA_AMODE_SINGLE_IDX       0x02
#define OMAP_DMA_AMODE_DOUBLE_IDX       0x03

/* Mapping of SDMA Request line with SDMA Request Number */
#define S_DMA_NO	0 /* Invalid DMA Request line */
#define S_DMA_0		1
#define S_DMA_1		2
#define S_DMA_2		3
#define S_DMA_3		4
#define S_DMA_4		5
#define S_DMA_5		6
#define S_DMA_6		7
#define S_DMA_7		8
#define S_DMA_8		9
#define S_DMA_9		10
#define S_DMA_10	11
#define S_DMA_11	12
#define S_DMA_12	13
#define S_DMA_13	14
#define S_DMA_14	15
#define S_DMA_15	16
#define S_DMA_16	17
#define S_DMA_17	18
#define S_DMA_18	19
#define S_DMA_19	20
#define S_DMA_20	21
#define S_DMA_21	22
#define S_DMA_22	23
#define S_DMA_23	24
#define S_DMA_24	25
#define S_DMA_25	26
#define S_DMA_26	27
#define S_DMA_27	28
#define S_DMA_28	29
#define S_DMA_29	30
#define S_DMA_30	31
#define S_DMA_31	32
#define S_DMA_32	33
#define S_DMA_33	34
#define S_DMA_34	35
#define S_DMA_35	36
#define S_DMA_36	37
#define S_DMA_37	38
#define S_DMA_38	39
#define S_DMA_39	40
#define S_DMA_40	41
#define S_DMA_41	42
#define S_DMA_42	43
#define S_DMA_43	44
#define S_DMA_44	45
#define S_DMA_45	46
#define S_DMA_46	47
#define S_DMA_47	48
#define S_DMA_48	49
#define S_DMA_49	50
#define S_DMA_50	51
#define S_DMA_51	52
#define S_DMA_52	53
#define S_DMA_53	54
#define S_DMA_54	55
#define S_DMA_55	56
#define S_DMA_56	57
#define S_DMA_57	58
#define S_DMA_58	59
#define S_DMA_59	60
#define S_DMA_60	61
#define S_DMA_61	62
#define S_DMA_62	63
#define S_DMA_63	64

/* System DMA request line mapping from Table 10.7 of  TRM */
#define OMAP_DMA_NO_DEVICE		S_DMA_NO
#define OMAP_DMA_XTI_DMA		S_DMA_0
#define OMAP_DMA_EXT_NDMA_REQ0		S_DMA_1
#define OMAP_DMA_EXT_NDMA_REQ1		S_DMA_2
#define OMAP_DMA_GPMC			S_DMA_3
#define OMAP_DMA_GFX			S_DMA_4
#define OMAP_DMA_DSS			S_DMA_5

#ifdef CONFIG_ARCH_OMAP2420
#define OMAP_DMA_VLYNQ_TX		S_DMA_6
#endif

#define OMAP_DMA_CWT			S_DMA_7
#define OMAP_DMA_AES_TX			S_DMA_8
#define OMAP_DMA_AES_RX			S_DMA_9
#define OMAP_DMA_DES_TX			S_DMA_10
#define OMAP_DMA_DES_RX			S_DMA_11

#ifdef CONFIG_ARCH_OMAP2420
#define OMAP_DMA_SHA1MD5_RX		S_DMA_12
#define OMAP_DMA_EAC_AC_RD		S_DMA_16
#define OMAP_DMA_EAC_AC_WR		S_DMA_17
#define OMAP_DMA_EAC_MD_UL_RD		S_DMA_18
#define OMAP_DMA_EAC_MD_UL_WR		S_DMA_19
#define OMAP_DMA_EAC_MD_DL_RD		S_DMA_20
#define OMAP_DMA_EAC_MD_DL_WR		S_DMA_21
#define OMAP_DMA_EAC_BT_UL_RD		S_DMA_22
#define OMAP_DMA_EAC_BT_UL_WR		S_DMA_23
#define OMAP_DMA_EAC_BT_DL_RD		S_DMA_24
#define OMAP_DMA_EAC_BT_DL_WR		S_DMA_25
#elif CONFIG_ARCH_OMAP2430
#define OMAP_DMA_SPI3_DMA_TX0		S_DMA_14
#define OMAP_DMA_SPI3_DMA_RX0		S_DMA_15
#define OMAP_DMA_MCBSP3_DMA_TX		S_DMA_16
#define OMAP_DMA_MCBSP3_DMA_RX		S_DMA_17
#define OMAP_DMA_MCBSP4_DMA_TX		S_DMA_18
#define OMAP_DMA_MCBSP4_DMA_RX		S_DMA_19
#define OMAP_DMA_MCBSP5_DMA_TX		S_DMA_20
#define OMAP_DMA_MCBSP5_DMA_RX		S_DMA_21
#define OMAP_DMA_SPI3_DMA_TX1		S_DMA_22
#define OMAP_DMA_SPI3_DMA_RX1		S_DMA_23
#endif

#define OMAP_DMA_I2C1_TX		S_DMA_26
#define OMAP_DMA_I2C1_RX		S_DMA_27
#define OMAP_DMA_I2C2_TX		S_DMA_28
#define OMAP_DMA_I2C2_RX		S_DMA_29
#define OMAP_DMA_MCBSP1_TX		S_DMA_30
#define OMAP_DMA_MCBSP1_RX		S_DMA_31
#define OMAP_DMA_MCBSP2_TX		S_DMA_32
#define OMAP_DMA_MCBSP2_RX		S_DMA_33
#define OMAP_DMA_SPI1_TX0		S_DMA_34
#define OMAP_DMA_SPI1_RX0		S_DMA_35
#define OMAP_DMA_SPI1_TX1		S_DMA_36
#define OMAP_DMA_SPI1_RX1		S_DMA_37
#define OMAP_DMA_SPI1_TX2		S_DMA_38
#define OMAP_DMA_SPI1_RX2		S_DMA_39
#define OMAP_DMA_SPI1_TX3		S_DMA_40
#define OMAP_DMA_SPI1_RX3		S_DMA_41
#define OMAP_DMA_SPI2_TX0		S_DMA_42
#define OMAP_DMA_SPI2_RX0		S_DMA_43
#define OMAP_DMA_SPI2_TX1		S_DMA_44
#define OMAP_DMA_SPI2_RX1		S_DMA_45

#ifdef CONFIG_ARCH_OMAP2430
#define OMAP_DMA_MMC2_TX		S_DMA_46
#define OMAP_DMA_MMC2_RX		S_DMA_47

#define OMAP_DMA_UART1_TX		S_DMA_48
#define OMAP_DMA_UART1_RX		S_DMA_49
#define OMAP_DMA_UART2_TX		S_DMA_50
#define OMAP_DMA_UART2_RX		S_DMA_51
#define OMAP_DMA_UART3_TX		S_DMA_52
#define OMAP_DMA_UART3_RX		S_DMA_53
#endif

#ifdef CONFIG_ARCH_OMAP2420
#define OMAP_DMA_UART1_TX		S_DMA_49
#define OMAP_DMA_UART1_RX		S_DMA_50
#define OMAP_DMA_UART2_TX		S_DMA_51
#define OMAP_DMA_UART2_RX		S_DMA_52
#define OMAP_DMA_UART3_TX		S_DMA_53
#define OMAP_DMA_UART3_RX		S_DMA_54
#endif

#define OMAP_DMA_USB_W2FC_TX0		S_DMA_54
#define OMAP_DMA_USB_W2FC_RX0		S_DMA_55
#define OMAP_DMA_USB_W2FC_TX1		S_DMA_56
#define OMAP_DMA_USB_W2FC_RX1		S_DMA_57
#define OMAP_DMA_USB_W2FC_TX2		S_DMA_58
#define OMAP_DMA_USB_W2FC_RX2		S_DMA_59
#define OMAP_DMA_MMC1_TX		S_DMA_60
#define OMAP_DMA_MMC1_RX		S_DMA_61
#define OMAP_DMA_MS			S_DMA_62

/* Register Base address */
#define OMAP_DMA4_BASE_PHYS             (OMAP24XX_L4_IO_BASE + 0x56000)
#define OMAP_DMA4_BASE_VIRT             IO_ADDRESS(OMAP_DMA4_BASE_PHYS)
#define OMAP_DMA4_BASE                  OMAP_DMA4_BASE_VIRT

#define OMAP_DMA4_GCR_REG               (OMAP_DMA4_BASE + 0x78)

#define OMAP_DMA4_IRQSTATUS_L0          (OMAP_DMA4_BASE + 0x08)
#define OMAP_DMA4_IRQSTATUS_L1          (OMAP_DMA4_BASE + 0x0c)
#define OMAP_DMA4_IRQSTATUS_L2          (OMAP_DMA4_BASE + 0x10)
#define OMAP_DMA4_IRQSTATUS_L3          (OMAP_DMA4_BASE + 0x14)

#define OMAP_DMA4_IRQENABLE_L0          (OMAP_DMA4_BASE + 0x18)
#define OMAP_DMA4_IRQENABLE_L1          (OMAP_DMA4_BASE + 0x1c)
#define OMAP_DMA4_IRQENABLE_L2          (OMAP_DMA4_BASE + 0x20)
#define OMAP_DMA4_IRQENABLE_L3          (OMAP_DMA4_BASE + 0x24)

#define OMAP_DMA4_SYSSTATUS             (OMAP_DMA4_BASE + 0x28)
#define OMAP_DMA4_OCP_SYSCONFIG         (OMAP_DMA4_BASE + 0x2C)

#define OMAP_DMA4_CAPS_0_REG            (OMAP_DMA4_BASE + 0x64)
#define OMAP_DMA4_CAPS_2_REG            (OMAP_DMA4_BASE + 0x6c)
#define OMAP_DMA4_CAPS_3_REG            (OMAP_DMA4_BASE + 0x70)
#define OMAP_DMA4_CAPS_4_REG            (OMAP_DMA4_BASE + 0x74)

/* Every LCh has its own set of the registers below */

#define OMAP_DMA4_CCR_REG(n)            (OMAP_DMA4_BASE + 0x60 * (n) + 0x80)
#define OMAP_DMA4_CLNK_CTRL_REG(n)      (OMAP_DMA4_BASE + 0x60 * (n) + 0x84)

#define OMAP_DMA4_CICR_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0x88)
#define OMAP_DMA4_CSR_REG(n)            (OMAP_DMA4_BASE + 0x60 * (n) + 0x8c)

#define OMAP_DMA4_CSDP_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0x90)
#define OMAP_DMA4_CEN_REG(n)            (OMAP_DMA4_BASE + 0x60 * (n) + 0x94)
#define OMAP_DMA4_CFN_REG(n)            (OMAP_DMA4_BASE + 0x60 * (n) + 0x98)

#define OMAP_DMA4_CSSA_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0x9c)
#define OMAP_DMA4_CDSA_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xa0)

#define OMAP_DMA4_CSEI_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xa4)
#define OMAP_DMA4_CSFI_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xa8)
#define OMAP_DMA4_CDEI_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xac)
#define OMAP_DMA4_CDFI_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xb0)

#define OMAP_DMA4_CSAC_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xb4)
#define OMAP_DMA4_CDAC_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xb8)
#define OMAP_DMA4_CCEN_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xbc)
#define OMAP_DMA4_CCFN_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xc0)

#define OMAP_DMA_COLOR_REG(n)           (OMAP_DMA4_BASE + 0x60 * (n) + 0xc4)

#define OMAP24XX_LOGICAL_DMA_CH_COUNT   32

typedef struct lch_chain_t {
	int num_channels;
	int queue_head;		/* head of the queue , this is the channel which is currently running */
	int queue_tail;		/* tail of the queue , this is the last channel which was queued */
	int started;
	int queued;
} lch_chain;
/* sync_mode
sync_mode	FS 	BS
	0x0	0 	0 	- an element is transferred once a dma
	0x1	0 	1	- an entire block is transferred once a dma
	0x2	1 	0	- an entire frame is transferred once a dma
	0x3	1 	1	- a packet is transferred once a dma
*/
/* Burst mode options */
enum omap_dma_burst_mode {
	OMAP_DMA_DATA_BURST_DIS = 0,
	OMAP_DMA_DATA_BURST_64,
	OMAP_DMA_DATA_BURST_32,
	OMAP_DMA_DATA_BURST_16
};
enum end_type {
	LITTLE_ENDIAN = 0,
	BIG_ENDIAN
};
enum omap_lch_loc {
	LC_SRC = 1,
	LC_DST,
	BOTH
};

typedef struct dma_channel_params_t {
	unsigned char read_prio;/* read priority */
	unsigned char write_prio;/* write priority */
	int data_type;		/* data type 8,16,32 */
	int elem_count;		/* number of elements in a frame */
	int frame_count;	/* number of frames in a element */

	int src_amode;		/* constant , post increment, indexed , double indexed */
	int src_start;		/* source address : physical */
	int src_ei;		/* source element index */
	int src_fi;		/* source frame index */

	int dst_amode;		/* constant , post increment, indexed , double indexed */
	int dst_start;		/* source address : physical */
	int dst_ei;		/* source element index */
	int dst_fi;		/* source frame index */

	int trigger;		/* trigger attached if the channel is synchronized */
	int sync_mode;		/* sycn on element, frame , block or packet */
	int src_or_dst_synch;	/* source synch(1) or destination synch(0) */

	int ie;			/* interrupt enabled */
	enum omap_dma_burst_mode burst_mode; /* Burst mode for lch: 16/32/64 bytes */
	enum omap_lch_loc b_loc; /* Src or Dest side for which burst to be enabled */
	enum omap_lch_loc e_loc; /* Src or Dest side for which endianism should to be enabled */
	enum end_type etype; 	 /* endian type for lch: BIG_ENDIAN/LITTLE_ENDIAN */
} dma_channel_params;

/* simple dma routines */
extern int omap_request_dma(int dev_id, const char *dev_name,
			void (* callback)(int lch, u16 ch_status, void *data),
			void *data, int *dma_ch_out);

extern void omap_free_dma(int ch);
extern int omap_start_dma(int lch);
extern int omap_stop_dma(int lch);

extern int omap_set_dma_transfer_params(int lch, int data_type, int elem_count,
			int frame_count, int sync_mode, int dma_trigger,
			int src_or_dst_synch);

extern int omap_set_dma_src_params(int lch, int src_amode,
			int src_start,int src_ei, int src_fi );

extern int omap_set_dma_dest_params(int lch, int dest_amode,
			int dest_start, int dst_ei, int dst_fi);

extern int omap_set_dma_params(int lch, dma_channel_params params);

int omap_get_dma_src_pos(int lch);
int omap_get_dma_dst_pos(int lch);
void omap_set_dma_dst_pos(int lch, int dest_addr);
int omap_clear_dma(int lch);

/* Chaining APIs */
int omap_request_dma_chain(int dev_id, const char *dev_name,
                 void (*callback) (int chain_id, u16 ch_status, void *data), int *chain_id,
                 int no_of_chans, int chain_mode, dma_channel_params params);
int omap_free_dma_chain(int chain_id);
int omap_dma_chain_a_transfer(int chain_id, int src_start, int dest_start, int elem_count,
			int frame_count, void *callbk_data);
int omap_start_dma_chain_transfers(int chain_id);
int omap_stop_dma_chain_transfers(int chain_id);
int omap_get_dma_chain_index(int chain_id, int *ei, int *fi);
int omap_get_dma_chain_dst_pos(int chain_id);
int omap_get_dma_chain_src_pos(int chain_id);

/* Priority APIs
 * one for setting global behavior
 * next for setting channel priority
 */
#define DMA_DEFAULT_FIFO_DEPTH     0x10
#define DMA_DEFAULT_ARB_RATE       0x01
/* Pass THREAD_RESERVE ORed with THREAD_FIFO for tparams */
#define DMA_THREAD_RESERVE_NORM   (0x00 << 12) /* Def */
#define DMA_THREAD_RESERVE_ONET   (0x01 << 12)
#define DMA_THREAD_RESERVE_TWOT   (0x02 << 12)
#define DMA_THREAD_RESERVE_THREET (0x03 << 12)
#define DMA_THREAD_FIFO_NONE      (0x00 << 14) /* Def */
#define DMA_THREAD_FIFO_75        (0x01 << 14)
#define DMA_THREAD_FIFO_25        (0x02 << 14)
#define DMA_THREAD_FIFO_50        (0x03 << 14)
/* Global prio settings - tparams is ignored for 242x */
void omap_dma_set_gprio_params(int arb_rate, int max_fifo_depth, int tparams);

#define DMA_CH_PRIO_HIGH           0x1
#define DMA_CH_PRIO_LOW            0x0 /* Def */
/* For 242x, read_prio is used for over all priority and write_prio is ignored */
int omap_dma_set_prio_lch(int lch, unsigned char read_prio, unsigned char write_prio);

void omap_set_dma_dest_burst_mode(int lch, enum omap_dma_burst_mode burst_mode);
void omap_set_dma_src_burst_mode(int lch, enum omap_dma_burst_mode burst_mode);

void omap_set_dma_dst_endian_type(int lch, enum end_type etype);
void omap_set_dma_src_endian_type(int lch, enum end_type etype);

/* index information retrieval apis */
int omap_get_dma_index(int lch, int *ei, int *fi);

/* Chaining modes*/
#define OMAP_DMA_STATIC_CHAIN 	0x1
#define OMAP_DMA_DYNAMIC_CHAIN 	0x2

int omap_dma_chain_status(int chain_id);
#define OMAP_DMA_CHAIN_ACTIVE   0x1
#define OMAP_DMA_CHAIN_INACTIVE 0x0

/*
 * Modify the chain's params - Modify the params after setting it
 * Dont do this while dma is running!!
 */
int omap_modify_dma_chain_params(int chain_id,dma_channel_params params);

#endif				/* __ASM_ARM_ARCH_OMAP2_DMA_H */
