#ifndef __ASM_ARCH_OMAP24XX_H
#define __ASM_ARCH_OMAP24XX_H

/*
 * Please place only base defines here and put the rest in device
 * specific headers. Note also that some of these defines are needed
 * for omap1 to compile without adding ifdefs.
 */

#define L4_24XX_BASE		0x48000000
#define L4_WK_243X_BASE		0x49000000
#define L3_24XX_BASE		0x68000000

/* interrupt controller */
#define OMAP24XX_IC_BASE	(L4_24XX_BASE + 0xfe000)
#define VA_IC_BASE		IO_ADDRESS(OMAP24XX_IC_BASE)
#define OMAP24XX_IVA_INTC_BASE	0x40000000
#define IRQ_SIR_IRQ		0x0040

#ifdef CONFIG_ARCH_OMAP2420
#define OMAP24XX_32KSYNCT_BASE	(L4_24XX_BASE + 0x4000)
#define OMAP24XX_PRCM_BASE	(L4_24XX_BASE + 0x8000)
#define OMAP24XX_SDRC_BASE	(L3_24XX_BASE + 0x9000)
#define OMAP242X_CONTROL_STATUS	(L4_24XX_BASE + 0x2f8)
#define OMAP2_CTRL_BASE		L4_24XX_BASE
#endif

#ifdef CONFIG_ARCH_OMAP2430
#define OMAP24XX_32KSYNCT_BASE	(L4_WK_243X_BASE + 0x20000)
#define OMAP24XX_PRCM_BASE	(L4_WK_243X_BASE + 0x6000)
#define OMAP242X_CONTROL_STATUS	(L4_24XX_BASE + 0x2f8)
#define OMAP243X_SMS_BASE	0x6C000000
#define OMAP24XX_SDRC_BASE	0x6D000000
#define OMAP243X_GPMC_BASE	0x6E000000
#define OMAP243X_HS_BASE	(L4_24XX_BASE + 0x000ac000)
#define OMAP24XX_L4_IO_BASE_MAP         (L4_24XX_BASE)
#define OMAP24XX_L4_IO_BASE             (OMAP24XX_L4_IO_BASE_MAP)
#define OMAP2_CTRL_BASE		(L4_WK_243X_BASE + 0x2000)
#endif

/* DSP SS */
#define OMAP24XX_DSP_BASE	0x58000000
#define OMAP24XX_DSP_MEM_BASE	(OMAP24XX_DSP_BASE + 0x0)
#define OMAP24XX_DSP_IPI_BASE	(OMAP24XX_DSP_BASE + 0x1000000)
#define OMAP24XX_DSP_MMU_BASE	(OMAP24XX_DSP_BASE + 0x2000000)

/* Mailbox */
#define OMAP24XX_MAILBOX_BASE	(L4_24XX_BASE + 0x94000)

/* GPMC */
#define GPMC_MAX_CS             7

/* GPMC regs offsets (32 bit regs) */
#define GPMC_OFF_CONFIG              0x50
#define GPMC_OFF_STATUS              0x54
#define GPMC_OFF_CS_0                0x60
#define GPMC_OFF_CONFIG1             0x00
#define GPMC_OFF_CONFIG2             0x04
#define GPMC_OFF_CONFIG3             0x08
#define GPMC_OFF_CONFIG4             0x0C
#define GPMC_OFF_CONFIG5             0x10
#define GPMC_OFF_CONFIG6             0x14
#define GPMC_OFF_CONFIG7             0x18
#define GPMC_OFF_NAND_COMMAND        0x1C
#define GPMC_OFF_NAND_ADDRESS        0x20
#define GPMC_OFF_NAND_DATA           0x24

#define OFFSET_CONFIG1_0          0x60
#define OFFSET_NAND_COMMAND_0     0x7C
#define OFFSET_NAND_ADDRESS_0     0x80
#define OFFSET_NAND_DATA_0        0x84
#define OFFSET_CONFIG             0x50
#define OFFSET_STATUS             0x54

#define GPMC_BUF_FULL                    0x00000001
#define GPMC_BUF_EMPTY                   0x00000000
#define NAND_WP_BIT                      0x00000010
#define WR_RD_PIN_MONITORING             0x00600000

#endif /* __ASM_ARCH_OMAP24XX_H */

