/*
 * include/asm-arm/arch-omap24xx/omap2-mcspi.h
 *
 * Driver Library for TI's Multichannled Serial Peripheral Interface
 *
 * Copyright (C) 2006 Texas Instruments, Inc.
 *
 * This package is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 *  History
 *  -------
 *  2006-06-10 Syed Mohammed Khasim     New file added to re-arch SPI driver
 */

#ifndef OMAP2_SPI_H
#define OMAP2_SPI_H

/* Bus Numbers */
#define OMAP2_McSPI_BUS0				0
#define OMAP2_McSPI_BUS1				1
#define OMAP2_McSPI_BUS2				2

/* Channel Numbers */
#define OMAP2_McSPI_CH0					0
#define OMAP2_McSPI_CH1					1
#define OMAP2_McSPI_CH2					2
#define OMAP2_McSPI_CH3					3

/* Bus operating mode selection */
#define OMAP2_McSPI_BUS_MASTER				0
#define OMAP2_McSPI_BUS_SLAVE				1

/* Single / Multi Channel on each Bus */
#define OMAP2_McSPI_BUS_SINGLECH			1
#define OMAP2_McSPI_BUS_MULTICH				0

#define OMAP2_McSPI_CHAN_DMA_READ			1
#define OMAP2_McSPI_CHAN_DMA_WRITE			1

/* Channel specific configuration */
#define OMAP2_McSPI_CHCFG_SPIENSLV_CS0			(0 << 19)
#define OMAP2_McSPI_CHCFG_SPIENSLV_CS1			(1 << 19)
#define OMAP2_McSPI_CHCFG_SPIENSLV_CS2			(2 << 19)
#define OMAP2_McSPI_CHCFG_SPIENSLV_CS3			(3 << 19)

/* DMA read/write request enable/disable */
#define OMAP2_McSPI_DMAW_ENABLE				1
#define OMAP2_McSPI_DMAW_DISABLE			0
#define OMAP2_McSPI_DMAR_ENABLE				1
#define OMAP2_McSPI_DMAR_DISABLE			0

/* Manual SPIEN assertion to keep SPIEN active between SPI words */
#define OMAP2_McSPI_CHCFG_ENABLE_FORCE			(1 << 20)
#define OMAP2_McSPI_CHCFG_DISABLE_FORCE			(0 << 20)

/* Input Selection */
#define OMAP2_McSPI_CHCFG_IS_DL0RECEIVE			(0 << 18)
#define OMAP2_McSPI_CHCFG_IS_DL1RECEIVE			(1 << 18)

/* Transmission Enable for data line 1 */
#define OMAP2_McSPI_CHCFG_DPE1_TRANSMIT			(0 << 17)
#define OMAP2_McSPI_CHCFG_DPE1_NO_TRANSMIT		(1 << 17)

/* Transmission Enable for data line 0 */
#define OMAP2_McSPI_CHCFG_DPE0_TRANSMIT			(0 << 16)
#define OMAP2_McSPI_CHCFG_DPE0_NO_TRANSMIT		(1 << 16)

/* Transmit / receive modes */
#define OMAP2_McSPI_CHCFG_TRANSRECEIVE			0
#define OMAP2_McSPI_CHCFG_RECEIVEONLY			1
#define OMAP2_McSPI_CHCFG_TRANSONLY			2

/* SPI word length */
#define OMAP2_McSPI_CHCFG_WL4				(0x03)
#define OMAP2_McSPI_CHCFG_WL5				(0x04)
#define OMAP2_McSPI_CHCFG_WL6				(0x05)
#define OMAP2_McSPI_CHCFG_WL7				(0x06)
#define OMAP2_McSPI_CHCFG_WL8				(0x07)
#define OMAP2_McSPI_CHCFG_WL9				(0x08)
#define OMAP2_McSPI_CHCFG_WL10				(0x09)
#define OMAP2_McSPI_CHCFG_WL11				(0x0A)
#define OMAP2_McSPI_CHCFG_WL12				(0x0B)
#define OMAP2_McSPI_CHCFG_WL13				(0x0C)
#define OMAP2_McSPI_CHCFG_WL14				(0x0D)
#define OMAP2_McSPI_CHCFG_WL15				(0x0E)
#define OMAP2_McSPI_CHCFG_WL16				(0x0F)
#define OMAP2_McSPI_CHCFG_WL17				(0x10)
#define OMAP2_McSPI_CHCFG_WL18				(0x11)
#define OMAP2_McSPI_CHCFG_WL19				(0x12)
#define OMAP2_McSPI_CHCFG_WL20				(0x13)
#define OMAP2_McSPI_CHCFG_WL21				(0x14)
#define OMAP2_McSPI_CHCFG_WL22				(0x15)
#define OMAP2_McSPI_CHCFG_WL23				(0x16)
#define OMAP2_McSPI_CHCFG_WL24				(0x17)
#define OMAP2_McSPI_CHCFG_WL25				(0x18)
#define OMAP2_McSPI_CHCFG_WL26				(0x19)
#define OMAP2_McSPI_CHCFG_WL27				(0x1A)
#define OMAP2_McSPI_CHCFG_WL28				(0x1B)
#define OMAP2_McSPI_CHCFG_WL29				(0x1C)
#define OMAP2_McSPI_CHCFG_WL30				(0x1D)
#define OMAP2_McSPI_CHCFG_WL31				(0x1E)
#define OMAP2_McSPI_CHCFG_WL32				(0x1F)

/* SPI "EN" polarity */
#define OMAP2_McSPI_CHCFG_EPOL_HIGH			(0 << 6)
#define OMAP2_McSPI_CHCFG_EPOL_LOW			(1 << 6)

/* Frequency divider for SPI clk */
#define OMAP2_McSPI_CHCFG_CLKD_1			(0x0)
#define OMAP2_McSPI_CHCFG_CLKD_2			(0x1)
#define OMAP2_McSPI_CHCFG_CLKD_4			(0x2)
#define OMAP2_McSPI_CHCFG_CLKD_8			(0x3)
#define OMAP2_McSPI_CHCFG_CLKD_16			(0x4)
#define OMAP2_McSPI_CHCFG_CLKD_32			(0x5)
#define OMAP2_McSPI_CHCFG_CLKD_64			(0x6)
#define OMAP2_McSPI_CHCFG_CLKD_128			(0x7)
#define OMAP2_McSPI_CHCFG_CLKD_256			(0x8)
#define OMAP2_McSPI_CHCFG_CLKD_512			(0x9)
#define OMAP2_McSPI_CHCFG_CLKD_1024			(0xA)
#define OMAP2_McSPI_CHCFG_CLKD_2048			(0xB)
#define OMAP2_McSPI_CHCFG_CLKD_4096			(0xC)
#define OMAP2_McSPI_CHCFG_CLKD_8192			(0xD)
#define OMAP2_McSPI_CHCFG_CLKD_16384			(0xE)
#define OMAP2_McSPI_CHCFG_CLKD_32768			(0xF)

/* SPI Clk Polarity */
#define OMAP2_McSPI_CHCFG_POL_HIGH			(0x0 << 1)
#define OMAP2_McSPI_CHCFG_POL_LOW			(0x1 << 1)

/* SPI CLK Phase */
#define OMAP2_McSPI_CHCFG_PHA_ODD			(0x0 << 0)
#define OMAP2_McSPI_CHCFG_PHA_EVEN			(0x1 << 0)

/* Interrupt handling */

#define OMAP2_McSPI_IRQ_WKS				(0x1 << 16)
#define OMAP2_McSPI_IRQ_RX_OVERFLOW			(0x1 << 3)
#define OMAP2_McSPI_IRQ_RX_FULL				(0x1 << 2)
#define OMAP2_McSPI_IRQ_TX_UNDERFLOW			(0x1 << 1)
#define OMAP2_McSPI_IRQ_TX_EMPTY			(0x1 << 0)

/************************************************************************/
/* Global dev Structure							*/
/************************************************************************/

/*
 * SPI channel configuration
 */
struct omap2_mcspi_dev {

	/* Channel Configurations */
	u32 phase;
	u32 polarity;
	u32 clk_divisor;
	u32 epol;
	u32 wordlen;
	u32 transmode;
	u32 dpe0;
	u32 dpe1;
	u32 is;
	u32 dma_write;
	u32 dma_read;

	/* Bus Configuration */
	u32 spi_chan;
	u32 spi_bus;
	u32 master;
	u32 single_multi;

	/* DMA callbacks */
	void (*dma_rx_callback) (struct omap2_mcspi_dev *, u32 status);
	void (*dma_tx_callback) (struct omap2_mcspi_dev *, u32 status);

	/* Can be used by client driver to keep track of any data on callback */
	void *dma_tx_cbdata;
	void *dma_rx_cbdata;
};

/*
 * The following macros can be used to fill up the omap2_mcspi_dev structure
 */

 /* set and get the phase */
#define omap2_mcspi_set_phase(dev, ph)			(dev.phase = ph)
#define omap2_mcspi_get_phase(dev)			(dev.phase)

/* set and get the data line polarity */
#define omap2_mcspi_set_polarity(dev, pol)		(dev.polarity = pol)
#define omap2_mcspi_get_polarity(dev)			(dev.polarity)

/* set and get the bus clock divisor */
#define omap2_mcspi_set_clkdivisor(dev, cd)		(dev.clk_divisor = cd)
#define omap2_mcspi_get_clkdivisor(dev)			(dev.clk_divisor)

/* set and get the clock polarity */
#define omap2_mcspi_set_epol(dev, pol)			(dev.epol = pol)
#define omap2_mcspi_get_epol(dev)			(dev.epol)

/* set and get the size of a single transfer */
#define omap2_mcspi_set_wordlen(dev, sz)		(dev.wordlen = sz)
#define omap2_mcspi_get_wordlen(dev)			(dev.wordlen)

/* set and get duplex: full or half */
#define omap2_mcspi_set_transmode(dev, mode)		(dev.transmode = mode)
#define omap2_mcspi_get_transmode(dev)			(dev.transmode)

/* set and get transmission enable for data line 0 */
#define omap2_mcspi_set_dpe0(dev, dp0)			(dev.dpe0 = dp0)
#define omap2_mcspi_get_dpe0(dev)			(dev.dpe0)

/* set and get transmission enable for data line 1 */
#define omap2_mcspi_set_dpe1(dev, dp1)			(dev.dpe1 = dp1)
#define omap2_mcspi_get_dpe1(dev)			(dev.dpe1)

/* set and get Input selection */
#define omap2_mcspi_set_is(dev, isv)			(dev.is = isv)
#define omap2_mcspi_get_is(dev)				(dev.is)

/* enable/ disable dma based read /write */
#define omap2_mcspi_set_dma_write(dev, dmawr)		(dev.dmaw = dmawr)
#define omap2_mcspi_set_dma_read(dev, dmard)		(dev.dmar = dmard)

#define omap2_mcspi_get_dma_write(dev)			(dev.dmaw)
#define omap2_mcspi_get_dma_read(dev)			(dev.dmar)

/* set the channel and bus combination */
#define omap2_mcspi_set_spi_chan(dev, chan)		(dev.spi_chan = chan)
#define omap2_mcspi_set_spi_bus(dev, bus)		(dev.spi_bus = bus)

#define omap2_mcspi_get_spi_chan(dev)			(dev.spi_chan)
#define omap2_mcspi_get_spi_bus(dev)			(dev.spi_bus)

/* set and get master/slave mode */
#define omap2_mcspi_set_master(dev, n)			(dev.master = n)
#define omap2_mcspi_get_master(dev)			(dev.master)

/* Interrupt handler */
typedef void (*omap2_mcspi_isr_t) (void *arg, u32 irq_status);

/* SPI based wakeup Interrupt handler */
typedef void (*omap2_mcspi_wakeupisr_t) (void *arg);

/************************************************************************/
/* Exported Functions 							*/
/************************************************************************/

/*
 * Allocate/Reserve an mcspi channel
 * This should be called after the configure macros above
 */
int omap2_mcspi_request_interface(struct omap2_mcspi_dev *dev);

/*
 * Release(unreserve) a mcspi channel
 * this deallocates the channel
 */
int omap2_mcspi_release_interface(struct omap2_mcspi_dev *dev);

/*
 * Software resets entire BUS
 */
int omap2_mcspi_interface_reset(struct omap2_mcspi_dev *dev);

/*
 *  A transfer that does not involve dma. You should
 *  call enable channel before and disable channel
 *  afterward
 */
int
 omap2_mcspi_raw_writetochannel(struct omap2_mcspi_dev *dev, u32 data);

/*
 *  Read that does not involve dma. You should
 *  call enable channel before and disable channel
 *  afterward
 */
int
 omap2_mcspi_raw_readfromchannel(struct omap2_mcspi_dev *dev, u32 * data);

/* This function is the same as omap2_mcspi_raw_readfromchannel, but
 * does not check the RX full bit in MCSPI_CH0STAT
 */
int
 omap2_mcspi_raw_readfromchannel_no_check(struct omap2_mcspi_dev *dev, u32 * data);

/*
 * Start reception of data in DMA mode (through DMA)
 * Enabling/Disabling a channel before/after DMA read
 * is the responsibility of client driver
 */
int omap2_mcspi_receive_buf(struct omap2_mcspi_dev *, dma_addr_t buf,
			    u32 size);

/*
 * Start transmission of data in DMA mode (through DMA)
 * Enabling/Disabling a channel before/after DMA read
 * is the responsibility of client driver
 */
int omap2_mcspi_transmit_buf(struct omap2_mcspi_dev *, dma_addr_t buf,
			     u32 bufsize);

/* Stop the on going DMA transfers */
int omap2_mcspi_stop_receive_buf(struct omap2_mcspi_dev *dev);
int omap2_mcspi_stop_transmit_buf(struct omap2_mcspi_dev *dev);

/*
 *  These functions increment(get) and decrement(put) a counter
 *  based on active use of the bus. This is used for power management. When the
 *  counter is 0 for a given spi bus, the bus is  suspended. When the
 *  counter then incremented, the bus is activated.
 *  It is crucial that these are used properly.
 */
int omap2_mcspi_put(struct omap2_mcspi_dev *dev);
int omap2_mcspi_get(struct omap2_mcspi_dev *dev);

/*
 * Register and unregister interrupts. The mask sets the interrupt as reads
 *  or writes.
 */
int omap2_mcspi_register_isr(struct omap2_mcspi_dev *dev,
			     omap2_mcspi_isr_t isr_callback, void *arg,
			     u32 mask);
int omap2_mcspi_unregister_isr(struct omap2_mcspi_dev *dev);

int omap2_mcspi_irqmask_cfg(struct omap2_mcspi_dev *dev, u32 mask);

/*
 * Wake up enable/disable API
 */

int omap2_mcspi_enable_wakeup(struct omap2_mcspi_dev *dev,
			      omap2_mcspi_wakeupisr_t wkisr_cb, void *arg);
int omap2_mcspi_disable_wakeup(struct omap2_mcspi_dev *dev);

/*
 * Enable or Disable a channel for raw reads or writes
 */
int omap2_mcspi_enable_channel(struct omap2_mcspi_dev *);
int omap2_mcspi_disable_channel(struct omap2_mcspi_dev *);

/*
 * configure the channel. This should be called directly
 * after a request_interface
 */
int omap2_mcspi_config_channel(struct omap2_mcspi_dev *);

/*
 * Used to print the SPI register values
 */
int omap2_mcspi_dump_regs(struct omap2_mcspi_dev *);

/* DMA read/write request Enable/Disable APIs */
int omap2_mcspi_dma_readreq(struct omap2_mcspi_dev *, u32 state);
int omap2_mcspi_dma_writereq(struct omap2_mcspi_dev *, u32 state);

/* Set SPI into active/autoidle */
int omap2_mcspi_set_active(struct omap2_mcspi_dev *);
int omap2_mcspi_set_autoidle(struct omap2_mcspi_dev *);

/* Set/clear FORCE bit in chanel config register */
int omap2_mcspi_set_force(struct omap2_mcspi_dev *);
int omap2_mcspi_clear_force(struct omap2_mcspi_dev *);

/* Disable interrupts */
int omap2_mcspi_disable_irq(struct omap2_mcspi_dev *dev);

#endif
